# -*- coding: utf-8 -*-
"""
    femagtools.losscoeffs
    ~~~~~~~~~~~~~~~~~~~~~

    Fitting methods for loss coeffs



"""
import numpy as np
import scipy.optimize as so


def pfe_jordan(f, B, ch, fh, cw, fw, fb, fo, Bo):
    return (ch*(f/fo)**fh + cw*(f/fo)**fw)*(B/Bo)**fb


def pfe_steinmetz(f, B, cw, fw, fb, fo, Bo):
    return cw*(f/fo)**fw * (B/Bo)**fb


def fitsteinmetz(f, B, losses, Bo, fo):
    """fit coeffs of
    losses(f,B)=cw*(f/fo)**alfa*(B/Bo)**beta
    returns (cw, alfa, beta)
    """
    pfe = np.asarray(losses).T
    z = []
    for i, fx in enumerate(f):
        if fx:
            if isinstance(B[0], float):
                z += [(fx, bx, y)
                      for bx, y in zip(B, pfe[i])
                      if isinstance(y, float)]
            else:
                z += [(fx, bx, y)
                      for bx, y in zip(B[i], pfe[i])
                      if y]

    fbx = np.array(z).T[0:2]
    y = np.array(z).T[2]

    fitp, cov = so.curve_fit(
        lambda x, cw, alpha, beta: pfe_steinmetz(
            x[0], x[1], cw, alpha, beta, fo, Bo),
        fbx, y, (1.0, 1.0, 2.0))
    return fitp


def fitjordan(f, B, losses, Bo, fo):
    """fit coeffs of
    losses(f,B)=(ch*(f/fo)**alpha + ch*(f/fo)**beta)*(B/Bo)**gamma
    returns (ch, alpha, cw, beta, gamma)
    """
    pfe = np.asarray(losses).T
    z = []
    for i, fx in enumerate(f):
        if fx:
            if isinstance(B[0], float):
                z += [(fx, bx, y)
                      for bx, y in zip(B, pfe[i])
                      if y]
            else:
                z += [(fx, bx, y)
                      for bx, y in zip(B[i], pfe[i])
                      if y]

    fbx = np.array(z).T[0:2]
    y = np.array(z).T[2]
    fitp, cov = so.curve_fit(lambda x, ch, alpha, cw, beta, gamma: pfe_jordan(
        x[0], x[1], ch, alpha, cw, beta, gamma, fo, Bo),
                             fbx, y, (1.0, 1.0, 1.0, 2.0, 1.0))
    return fitp



# -*- coding: utf-8 -*-
"""
    femagtools.airgap
    ~~~~~~~~~~~~~~~~~

    Read airgap dat file



"""
import numpy as np
import logging

logger = logging.getLogger(__name__)


def read(filename, pmod):
    """read dat file with columns (phi, Br, Bphi)
    returns samples, values, amplitude and phase of base harmonic

    Args:
      filename: the name of the file to be processed
      pmod: number of poles in model
    """
    bag = np.loadtxt(filename).T
    if len(bag) < 3:
        logger.warn("%s has incomplete content", filename)
        return(dict())

    N = 2**10  # The DFT is most efficient when N is a power of 2
    phi = np.linspace(0, 2*np.pi, N)
    nphi = int(round(360/((bag[0][-1] - bag[0][0])/(len(bag[0]) - 1)))) + 1
    ntiles = (nphi-1)//(len(bag[0]) - 1)
    phitab = np.linspace(0, 2*np.pi, nphi)
    if pmod % 2:
        bx = np.append(
            np.concatenate(
                [n*bag[1][:-1]
                 for n in [m % 2 or -1
                           for m in range(1, ntiles+1)]]),
            bag[1][0])
    else:
        bx = np.append(
            np.tile(bag[1][:-1], ntiles),
            bag[1][0])

    br = np.interp(phi, phitab, bx)
    npoles = ntiles*pmod

    # compute DFT from induction
    Y = np.fft.fft(br)
    freq = np.fft.fftfreq(N, d=phi[1]-phi[0])
    
    # find the peak (amplitude of base harmonic)
    i = np.argmax(np.abs(Y[:N//2]))
    a = 2*np.abs(Y[i])/N
    T0 = np.abs(1/freq[i])
    logger.info("%s: %s poles B amp %f ",
                filename, npoles, a)

    alfa0 = np.angle(Y[i])
    alfa = bag[0]/180*np.pi

    return dict(Bamp=a,
                phi0=alfa0,
                pos=bag[0].tolist(),
                B=bag[1].tolist(),
                nue=np.arange(0, 9*npoles).tolist(),
                B_nue=(2*np.abs(Y[:9*npoles])/N).tolist(),
                B_fft=(a*np.cos(2*np.pi*alfa/T0+alfa0)).tolist())



# -*- coding: utf-8 -*-
"""
    femagtools.gmsh
    ~~~~~~~~~~~~~~~~

    Convert FEMAG modelfile to gmsh


"""
import femagtools.isa7
import logging
import os
import sys

if __name__ == "__main__":
    
    logging.basicConfig(level=logging.INFO,
                        format='%(asctime)s %(message)s')
    if len(sys.argv) == 2:
        filename = sys.argv[1]
    else:
        filename = sys.stdin.readline().strip()

    isa = femagtools.isa7.read(filename)

    basename = os.path.splitext(os.path.basename(filename))[0]
    with open(basename + '.msh', 'w') as f:
        f.write('\n'.join(isa.msh()))



'''
  jcf2msh Convert JMag JCF Files to Gmsh/msh

  Usage:

    jcf2msh <infile> <outfile>

  Author: Jonas Knöpfel

'''
import argparse
import re
from zipfile import ZipFile
from collections import namedtuple

parser = argparse.ArgumentParser()
parser.add_argument("infile")
parser.add_argument("outfile")

args = parser.parse_args()

infile = args.infile
outfile = args.outfile

with ZipFile(infile, 'r') as jcf:
    with jcf.open("mesh.dat") as f:
        f = [l.decode("utf-8") for l in f.readlines()]
        f = iter(f)
        
def next_line(f, n=1):
    for i in range(n):
        line = next(f)
    line = line.strip()
    line = re.split(" +|,", line)
    return line


Node = namedtuple("Node", ["id", "x", "y"])
nodes = []

Element = namedtuple("Element", ["nodes", "entity"])
elements = []

while True:
    try:
        line = next_line(f)
        
        if line[-1] == "*Node":
            for n in range(int(line[0])):
                line = next_line(f)
                nodes.append(Node(line[1], line[2], line[3]))
                
            replace = {}
            for i, n1 in enumerate(nodes):
                for n2 in nodes[i+1:]:
                    if n1.x == n2.x and n1.y == n2.y and n1.id != n2.id:
                        if n1.id not in replace:
                            replace[n2.id] = n1.id
                            
            nodes = [n for n in nodes if n not in replace]
                
        if line[-1] == "*Element":
            for n in range(int(line[0])):
                line = next_line(f)
                if line[1] == "3":
                    elements.append(
                        Element([line[-3], line[-2], line[-1], None], line[2]))
                elif line[1] == "4":
                    elements.append(
                        Element([line[-4], line[-3], line[-2], line[-1]], line[2]))
                    
            for e in elements:
                for i, n in enumerate(e.nodes):
                    if n in replace:
                        e.nodes[i] = replace[n]
                    
    except StopIteration:
        break

msh = ["$MeshFormat\n2.2 0 8\n$EndMeshFormat"]

msh.append("$Nodes")
msh.append(str(len(nodes)))
for i, n in enumerate(nodes):
    msh.append("{} {} {} 0.0".format(n.id, n.x, n.y))
msh.append("$EndNodes")

msh.append("$Elements")
msh.append(str(len(elements)))
for i, e in enumerate(elements):
    if e.nodes[3]:
        msh.append("{} 3 2 0 {} {} {} {} {}".format(
            i + 1, e.entity, e.nodes[0], e.nodes[1], e.nodes[2], e.nodes[3]))
    else:
        msh.append("{} 2 2 0 {} {} {} {}".format(
            i + 1, e.entity, e.nodes[0], e.nodes[1], e.nodes[2]))
msh.append("$EndElements")
               
with open(outfile, "w") as outfile:
    outfile.write("\n".join(msh))
